
#ifndef __RSHAPER_SYSDEP_H__
#define __RSHAPER_SYSDEP_H__

#ifdef __KERNEL__ /* system dependecies are only in kernel space */

#include <linux/config.h>
#ifndef UTS_RELEASE
#  include <linux/version.h>
#endif

#ifndef KERNEL_VERSION
#  define KERNEL_VERSION(vers,rel,seq) ( ((vers)<<16) | ((rel)<<8) | (seq) )
#endif

/* Only run with 2.0, 2.2, 2.4 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,0,0) /* not < 2.0 */
#  error "This kernel is too old: not supported by this file"
#  error LINUX_VERSION_CODE
#endif
#if (LINUX_VERSION_CODE & 0xff00) == 0x100 /* not 2.1 */
#  error "Please don't use linux-2.1, use 2.2 instead"
#endif
#if (LINUX_VERSION_CODE & 0xff00) == 0x300 /* not 2.3 */
#  error "Please don't use linux-2.3, use 2.4 instead"
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0) /* not > 2.4 */
#  error "Kernel too recent, this module won't run"
#endif

#undef LINUX_24
#undef LINUX_22
#undef LINUX_20
/* remember about the current version */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
#  define LINUX_24
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)
#  define LINUX_22
#else
#  define LINUX_20
#endif


#ifdef LINUX_24 /* 2.4 uses netfilter */
#  ifndef CONFIG_NETFILTER
#    error "Netfilter support is not enabled in kernel configuration"
#  endif
#  include <linux/netfilter.h>
#  include <linux/netfilter_ipv4.h>

/* map our functions to netfilter stuff */
#  define rshaper_drop_fun(skb)		do {} while (0)   /* do nothing */
#  define rshaper_accept_fun(skb)	do {} while (0)   /* do nothing */

#else

/* map them directly */
#  define rshaper_drop_fun(skb)		kfree_skb(skb)
#  define rshaper_accept_fun(skb)	netif_rx(skb)

/* These do not exist in pre-2.4 */
#  define NF_STOLEN	0
#  define NF_DROP	0
#  define NF_ACCEPT	0

#endif /* 2.4 */

#ifndef LINUX_24
#  define net_device device   /* Before 2.4 `net_device' struct was `device' */
#endif

#ifdef LINUX_20
#  include <asm/segment.h>
#  define access_ok(t,a,sz)           (verify_area((t),(a),(sz)) ? 0 : 1)
#  define verify_area_20              verify_area
#  define copy_to_user(t,f,n)         (memcpy_tofs(t,f,n), 0)
#  define copy_from_user(t,f,n)       (memcpy_fromfs((t),(f),(n)), 0)
#  define kfree_skb(skb) ({(skb)->free = 1; kfree_skb((skb), 0);})
#else
#  include <asm/uaccess.h>
#  include <asm/io.h>
#  define verify_area_20(t,a,sz) (0) /* == success */
#endif

/*
 * In 2.0, there is no real need for spinlocks, and they weren't really
 * implemented anyway.
 */
#ifdef LINUX_20
typedef int spinlock_t;
#  define spin_lock(lock)
#  define spin_unlock(lock)
#  define spin_lock_init(lock)
 
#  define spin_lock_irqsave(lock,flags) do { \
        save_flags(flags); cli(); } while (0);
#  define spin_unlock_irqrestore(lock,flags) restore_flags(flags);
#endif

/* No del_timer_sync before 2.4 */
#ifndef LINUX_24
#  define del_timer_sync(timer) del_timer(timer)  /* and hope */
#endif


#endif /* __KERNEL__ */
#endif /* __RSHAPER_SYSDEP_H__ */
